@@ -1,10 +1,9 @@
 """Buildout Versions Checker"""
-from bvc.cmdline import cmdline
 from bvc.checker import VersionsChecker
 from bvc.configparser import VersionsConfigParser
+from bvc.scripts.check_buildout_updates import cmdline
 
 __all__ = [
-    cmdline.__name__,
     VersionsChecker.__name__,
     VersionsConfigParser.__name__
 ]
